@@ -1,5 +1,6 @@
 #include <stdio.h>
 #include <stdlib.h>
+#include <locale.h>
 #include <verbiste/c-api.h>
 
 
@@ -37,8 +38,8 @@ demo(const char *utf8_infinitive)
             }
 
             printf("- %s %s:\n",
-                (const char*) verbiste_get_mode_name(mode),
-                (const char*) verbiste_get_tense_name(tense));
+                (const char *) verbiste_get_mode_name(mode),
+                (const char *) verbiste_get_tense_name(tense));
 
             for (i = 0; person_array[i] != NULL; i++)
             {
@@ -77,6 +78,7 @@ int main( int argc, char *argv[])
         return EXIT_FAILURE;
     }
 
+    setlocale(LC_CTYPE, "");  // necessary on Solaris
     libdatadir = getenv("LIBDATADIR");
     if (libdatadir == NULL)
         libdatadir = LIBDATADIR;
@@ -84,7 +86,11 @@ int main( int argc, char *argv[])
     snprintf(conjFN, sizeof(conjFN), "%s/conjugation-fr.xml", libdatadir);
     snprintf(verbsFN, sizeof(verbsFN), "%s/verbs-fr.xml", libdatadir);
 
-    verbiste_init(conjFN, verbsFN, "fr");
+    if (verbiste_init(conjFN, verbsFN, "fr") != 0)
+    {
+        printf("conjugator.c: failed to initialize Verbiste.\n");
+        return EXIT_FAILURE;
+    }
 
     exit_status = demo(argv[1]);
 
