@@ -5,7 +5,7 @@ void dummy()
     using namespace verbiste;
 
     FrenchVerbDictionary fvd0(true);
-    FrenchVerbDictionary fvd1("", "", true, FrenchVerbDictionary::GREEK);
+    FrenchVerbDictionary fvd1("", "", true, FrenchVerbDictionary::FRENCH);
 }
 
 int main()
