@@ -1,8 +1,8 @@
-/*  $Id: main-window.cpp,v 1.46 2014/04/06 23:30:40 sarrazip Exp $
+/*  $Id: main-window.cpp,v 1.48 2019/07/28 17:30:24 sarrazip Exp $
     main-window.cpp - Input and conjugation window
 
     verbiste - French conjugation system
-    Copyright (C) 2003-2014 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2003-2019 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -446,21 +446,104 @@ esc_latex(const string &s)
 }
 
 
-void writeHTMLConjugation(ostream &html)
+class ConjugationWriter
 {
-    html << "<?xml version='1.0'?>\n";
-    html << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
-            << " \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
-    html << "<html>\n";
-    html << "<head>\n";
-    html << "<title>" << esc_html(gtk_entry_get_text(GTK_ENTRY(verbEntry)))
-            << "</title>\n";
-    html << "<meta http-equiv='Content-Type'"
-            << " content='text/html; charset=UTF-8' />\n";
-    html << "</head>\n";
-    html << "<body>\n";
+public:
+    ConjugationWriter(ostream &_out) : out(_out) {}
+    virtual ~ConjugationWriter() {}
+    virtual void beginDocument(const char *title) = 0;
+    virtual void endDocument() = 0;
+    virtual void addComment(const string &text) = 0;
+    virtual void writeTitle(const char *title) = 0;
+    virtual void beginTable() = 0;
+    virtual void endTable() = 0;
+    virtual void beginRow() = 0;
+    virtual void endRow() = 0;
+    virtual void beginCell() = 0;
+    virtual void endCell() = 0;
+    virtual void addEmptyCell() = 0;
+    virtual void addPersons(const char *text) = 0;
+    virtual void addTenseName(const char *text) = 0;
+protected:
+    ostream &out;
+};
 
 
+class HTMLConjugationWriter : public ConjugationWriter
+{
+public:
+    HTMLConjugationWriter(ostream &_out) : ConjugationWriter(_out) {}
+    virtual void beginDocument(const char *documentTitle)
+    {
+        out << "<?xml version='1.0'?>\n";
+        out << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
+                << " \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
+        out << "<html>\n";
+        out << "<head>\n";
+        out << "<title>" << esc_html(documentTitle) << "</title>\n";
+        out << "<meta http-equiv='Content-Type'"
+                << " content='text/html; charset=UTF-8' />\n";
+        out << "</head>\n";
+        out << "<body>\n";
+    }
+    virtual void endDocument()
+    {
+        out << "</body>\n";
+        out << "</html>\n";
+    }
+    virtual void addComment(const string &text)
+    {
+        out << "<!-- " << esc_html(text.c_str()) << " -->\n";
+    }
+    virtual void writeTitle(const char *title)
+    {
+        out << "<h1>" << esc_html(title) << "</h1>\n";
+    }
+    virtual void beginTable()
+    {
+        out << "<table border='1' cellspacing='0' cellpadding='4'>\n";
+    }
+    virtual void endTable()
+    {
+        out << "</table>\n";
+    }
+    virtual void beginRow()
+    {
+        out << "<tr valign='top'>\n";
+    }
+    virtual void endRow()
+    {
+        out << "</tr>\n";
+    }
+    virtual void beginCell()
+    {
+        out << "<td>\n";
+    }
+    virtual void endCell()
+    {
+        out << "</td>\n";
+    }
+    virtual void addEmptyCell()
+    {
+        out << "<td>&nbsp;</td>\n";
+    }
+    virtual void addTenseName(const char *text)
+    {
+        out << "<strong>" << esc_html(text) << "</strong><br/>";
+    }
+    virtual void addPersons(const char *text)
+    {
+        out << esc_html(text);
+    }
+};
+
+
+void writeConjugation(ConjugationWriter &writer)
+{
+    writer.addComment("Generated by " + string(PACKAGE_FULL_NAME) + " " + VERSION);
+
+    writer.beginDocument(gtk_entry_get_text(GTK_ENTRY(verbEntry)));
+
     assert(resultNotebook != NULL);
     GtkWidget *w;
     for (int i = 0; (w = gtk_notebook_get_nth_page(
@@ -478,175 +561,190 @@ void writeHTMLConjugation(ostream &html)
         g_return_if_fail(GTK_IS_LABEL(label));
 
         const gchar *infinitive = gtk_label_get_text(GTK_LABEL(label));
-        html << "<h1>" << esc_html(infinitive) << "</h1>\n";
-        html << "<table border='1' cellspacing='0' cellpadding='4'>\n";
-        string tr = "<tr valign='top'>\n";
-        html << tr;
+        writer.writeTitle(infinitive);
+        writer.beginTable();
+        writer.beginRow();
 
         GList *children = gtk_container_get_children(GTK_CONTAINER(rp->table));
         children = g_list_reverse(children);
         int childCounter = 0;
         for (GList *p = children; p != NULL; p = p->next, childCounter++)
         {
-            GtkWidget *vbox = (GtkWidget *) p->data;
-            g_return_if_fail(vbox != NULL);
-            g_return_if_fail(GTK_IS_VBOX(vbox));
-
-            GList *vboxChildren = gtk_container_get_children(
-                                                GTK_CONTAINER(vbox));
-            g_return_if_fail(vboxChildren != NULL);
-            g_return_if_fail(vboxChildren->next != NULL);
-            g_return_if_fail(vboxChildren->next->next == NULL);
-
-            GtkWidget *nameLabel = (GtkWidget *) vboxChildren->data;
-            GtkWidget *personsLabel = (GtkWidget *) vboxChildren->next->data;
-
-            g_return_if_fail(nameLabel != NULL);
-            g_return_if_fail(GTK_IS_LABEL(nameLabel));
-            g_return_if_fail(personsLabel != NULL);
-            g_return_if_fail(GTK_IS_LABEL(personsLabel));
-
-            const gchar *name = gtk_label_get_text(GTK_LABEL(nameLabel));
-            const gchar *persons = gtk_label_get_text(GTK_LABEL(personsLabel));
-
-            g_return_if_fail(name != NULL);
-            g_return_if_fail(persons != NULL);
-
-            html << "<td>\n";
-            html << "<strong>" << esc_html(name) << "</strong>\n<br/>\n";
-            html << esc_html(persons) << "\n";
-            html << "</td>\n";
-
-            if (childCounter == 0)
+            GtkWidget *widget = (GtkWidget *) p->data;
+            g_return_if_fail(widget != NULL);
+            
+            if (GTK_IS_VBOX(widget))
             {
-                html << "<td>&nbsp;</td>\n";
-                html << "<td>&nbsp;</td>\n";
-                html << "<td>&nbsp;</td>\n";
-                html << "</tr>\n";
-                html << tr;
-            }
-            else if (childCounter == 4)
-            {
-                html << "</tr>\n";
-                html << tr;
-            }
-            else if (childCounter == 7 || childCounter == 10)
-            {
-                html << "<td>&nbsp;</td>\n";
-                html << "</tr>\n";
-                if (childCounter == 7)
-                    html << tr;
+                GList *vboxChildren = gtk_container_get_children(
+                                                    GTK_CONTAINER(widget));
+                g_return_if_fail(vboxChildren != NULL);
+                g_return_if_fail(vboxChildren->next != NULL);
+                g_return_if_fail(vboxChildren->next->next == NULL);
+
+                GtkWidget *nameLabel = (GtkWidget *) vboxChildren->data;
+                GtkWidget *personsLabel = (GtkWidget *) vboxChildren->next->data;
+
+                g_return_if_fail(nameLabel != NULL);
+                g_return_if_fail(GTK_IS_LABEL(nameLabel));
+                g_return_if_fail(personsLabel != NULL);
+                g_return_if_fail(GTK_IS_LABEL(personsLabel));
+
+                const gchar *name = gtk_label_get_text(GTK_LABEL(nameLabel));
+                const gchar *persons = gtk_label_get_text(GTK_LABEL(personsLabel));
+
+                g_return_if_fail(name != NULL);
+                g_return_if_fail(persons != NULL);
+
+                writer.beginCell();
+                writer.addTenseName(name);
+                writer.addPersons(persons);
+                writer.endCell();
+
+                g_list_free(vboxChildren);
             }
 
-            g_list_free(vboxChildren);
+            if (childCounter == 2)  // start new row after inf. pres.
+            {
+                writer.addEmptyCell();
+                writer.addEmptyCell();
+                writer.addEmptyCell();
+                writer.endRow();
+                writer.beginRow();
+            }
+            else if (childCounter == 6)  // start new row after ind. past.
+            {
+                writer.endRow();
+                writer.beginRow();
+            }
+            else if (childCounter == 9)  // start new row after subj. imperf.
+            {
+                writer.addEmptyCell();
+                writer.endRow();
+                writer.beginRow();
+            }
         }
         g_list_free(children);
 
-        html << "</table>\n";
+        writer.endRow();
+        writer.endTable();
     }
 
-    html << "</body>\n";
-    html << "</html>\n";
+    writer.endDocument();
 }
 
 
-static
-void
-writeLaTeXConjugation(ostream &tex)
+class LaTeXConjugationWriter : public ConjugationWriter
 {
-    tex << "% verbiste output - do not edit\n";
-    tex << "% " << esc_latex(gtk_entry_get_text(GTK_ENTRY(verbEntry))) << "\n";
-    tex << "\\documentclass{verbiste}\n";
-    tex << "\\title{" << esc_latex(gtk_entry_get_text(GTK_ENTRY(verbEntry))) << "}\n";
-    tex << "\\begin{document}\n";
-    tex << "\\thispagestyle{empty}\n";
-
-    assert(resultNotebook != NULL);
-    GtkWidget *w;
-    for (int i = 0; (w = gtk_notebook_get_nth_page(
-                        GTK_NOTEBOOK(resultNotebook), i)) != NULL;
-                    i++)
+public:
+    LaTeXConjugationWriter(ostream &_out) : ConjugationWriter(_out), numCellsInRow(0) {}
+    virtual void beginDocument(const char *documentTitle)
     {
-        ResultPage *rp = (ResultPage *) g_object_get_data(
-                                                G_OBJECT(w), "ResultPage");
-        g_return_if_fail(rp != NULL);
-        g_return_if_fail(GTK_IS_TABLE(rp->table));
-
-        GtkWidget *label = gtk_notebook_get_tab_label(
-                                            GTK_NOTEBOOK(resultNotebook),
-                                            rp->notebookPage);
-        g_return_if_fail(GTK_IS_LABEL(label));
-
-        string btab = "\\begin{tense}\n";
-        string etab = "\\end{tense}\n";
-        string eol  = "\\\\\n";
-        string eor  = "\\\\ \\hline\n";
-        string sep  = "&\n";
-
-        const gchar *infinitive = gtk_label_get_text(GTK_LABEL(label));
-        tex << "\\section*{" << esc_latex(infinitive) << "}\n";
-        tex << "\\noindent\n";
-        tex << "\\begin{tensetable}\n";
-
-        GList *children = gtk_container_get_children(GTK_CONTAINER(rp->table));
-        children = g_list_reverse(children);
-        int childCounter = 0;
-        for (GList *p = children; p != NULL; p = p->next, childCounter++)
-        {
-            GtkWidget *vbox = (GtkWidget *) p->data;
-            g_return_if_fail(vbox != NULL);
-            g_return_if_fail(GTK_IS_VBOX(vbox));
-
-            GList *vboxChildren = gtk_container_get_children(
-                                                GTK_CONTAINER(vbox));
-            g_return_if_fail(vboxChildren != NULL);
-            g_return_if_fail(vboxChildren->next != NULL);
-            g_return_if_fail(vboxChildren->next->next == NULL);
-
-            GtkWidget *nameLabel = (GtkWidget *) vboxChildren->data;
-            GtkWidget *personsLabel = (GtkWidget *) vboxChildren->next->data;
-
-            g_return_if_fail(nameLabel != NULL);
-            g_return_if_fail(GTK_IS_LABEL(nameLabel));
-            g_return_if_fail(personsLabel != NULL);
-            g_return_if_fail(GTK_IS_LABEL(personsLabel));
-
-            const gchar *name = gtk_label_get_text(GTK_LABEL(nameLabel));
-            const gchar *persons = gtk_label_get_text(GTK_LABEL(personsLabel));
-
-            g_return_if_fail(name != NULL);
-            g_return_if_fail(persons != NULL);
-
-            tex << btab;
-            tex << "\\tensename{" << esc_latex(name) << "}" << eol;
-            tex << esc_latex(persons) << "\n";
-            tex << etab;
-
-            switch (childCounter)
-              {
-              case 0:
-                tex << sep << sep << sep << eor;
-                break;
-              case 4:
-                tex << eor;
-                break;
-              case 7:
-              case 10:
-                tex << sep << eor;
-                break;
-              default:
-                tex << sep;
-              }
-
-            g_list_free(vboxChildren);
-        }
-        g_list_free(children);
-
-        tex << "\\end{tensetable}\n";
+        out << "\\documentclass{article}\n";
+        
+        out <<
+            "\n"
+            "\\RequirePackage[utf8]{inputenc}\n"
+            "\n"
+            "\\newif\\ifvlines \\vlinesfalse\n"
+            "\\newif\\ifhlines \\hlinesfalse\n"
+            "\n"
+            "% handles the spacing above and below the tense name\n"
+            "% and below the persons of the tense\n"
+            "\n"
+            "\\newcommand\\topstrut{\\rule{0pt}{3.5ex}}\n"
+            "\\newcommand\\midstrut{\\rule[-1ex]{0pt}{0pt}}\n"
+            "\\newcommand\\botstrut{\\rule[-2ex]{0pt}{0pt}}\n"
+            "\n"
+            "% the tense name\n"
+            "\n"
+            "\\newcommand{\\tensename}[1]{%\n"
+            "  \\textbf{#1}\\topstrut\\midstrut}\n"
+            "\n"
+            "% the tense environment\n"
+            "\n"
+            "\\newenvironment{tense}{%\n"
+            "  \\begin{tabular}[t]{l}}{\\botstrut\\end{tabular}}\n"
+            "\n"
+            "% tense table horizontal lines\n"
+            "\n"
+            "\\newcommand{\\vbhl}{%\n"
+            "  \\ifhlines\\hline\\else\\relax\\fi}\n"
+            "\n"
+            "% the tensetable environment\n"
+            "\n"
+            "\\newenvironment{tensetable}{%\n"
+            "  \\begin{tabular}{%\n"
+            "      \\ifvlines \n"
+            "      |l|l|l|l|l|\n"
+            "      \\else \n"
+            "      llll\n"
+            "      \\fi}\\vbhl\n"
+            "}{\\end{tabular}}\n"
+            "\n"
+            ;
+        
+        out << "\\title{" << esc_latex(documentTitle) << "}\n";
+        out << "\\begin{document}\n";
+        out << "\\thispagestyle{empty}\n";
     }
-
-    tex << "\\end{document}\n";
-}
+    virtual void endDocument()
+    {
+        out << "\\end{document}\n";        
+    }
+    virtual void addComment(const string &text)
+    {
+        out << "% " << esc_latex(text) << "\n";
+    }
+    virtual void writeTitle(const char *title)
+    {
+        out << "\\section*{" << esc_latex(title) << "}\n";
+    }
+    virtual void beginTable()
+    {
+        out << "\\noindent\n";
+        out << "\\begin{tensetable}\n";
+    }
+    virtual void endTable()
+    {
+        out << "\\end{tensetable}\n";
+    }
+    virtual void beginRow()
+    {
+        numCellsInRow = 0;
+    }
+    virtual void endRow()
+    {
+        out << "\\\\ \\hline\n";
+    }
+    virtual void beginCell()
+    {
+        if (numCellsInRow > 0)
+            out << "&\n";
+        out << "\\begin{tense}\n";
+    }
+    virtual void endCell()
+    {
+        out << "\\end{tense}\n";
+        ++numCellsInRow;
+    }
+    virtual void addEmptyCell()
+    {
+        if (numCellsInRow > 0)
+            out << "&\n";
+        out << "\n";
+        ++numCellsInRow;
+    }
+    virtual void addTenseName(const char *text)
+    {
+        out << "\\tensename{" << esc_latex(text) << "}\\\\\n";
+    }
+    virtual void addPersons(const char *text)
+    {
+        out << esc_latex(text) << "\n";
+    }
+private:
+    size_t numCellsInRow;
+};
 
 
 enum SaveFileFormat { HTML, LATEX, NUMFORMATS };
@@ -757,7 +855,8 @@ onSaveAsButton(GtkWidget *, gpointer)
     gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(chooser), defaultFilename.c_str());
 
 
-    bool cancel = (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_CANCEL);
+    gint response = gtk_dialog_run(GTK_DIALOG(chooser));
+    bool cancel = (response != GTK_RESPONSE_ACCEPT);  // escape key may return GTK_RESPONSE_DELETE_EVENT
     string filename;
     SaveFileFormat chosenFormat = NUMFORMATS;  // invalid value for now
     if (!cancel)
@@ -794,11 +893,17 @@ onSaveAsButton(GtkWidget *, gpointer)
     switch (chosenFormat)
     {
         case HTML:
-            writeHTMLConjugation(file);
+        {
+            HTMLConjugationWriter writer(file);
+            writeConjugation(writer);
             break;
+        }
         case LATEX:
-            writeLaTeXConjugation(file);
+        {
+            LaTeXConjugationWriter writer(file);
+            writeConjugation(writer);
             break;
+        }
         case NUMFORMATS:
             assert(!"invalid chosen format");
             break;
