@@ -1,8 +1,8 @@
-/*  $Id: conjugation.cpp,v 1.12 2011/01/25 02:29:44 sarrazip Exp $
+/*  $Id: conjugation.cpp,v 1.15 2013/06/28 18:12:03 sarrazip Exp $
     conjugation.cpp - Generic conjugation interface
 
     verbiste - French conjugation system
-    Copyright (C) 2003-2006 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2003-2013 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -16,8 +16,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
 #include "conjugation.h"
@@ -122,7 +122,8 @@ static const char *tn[16] =
 };
 
 
-string getTenseNameForTableCell(int row, int col, bool isItalian)
+string
+getTenseNameForTableCell(int row, int col, bool isItalian)
 {
     if (row < 0 || row > 3 || col < 0 || col > 3)
     {
@@ -158,10 +159,9 @@ removePronoun(const string &s)
 
     for (size_t i = 0; pronouns[i] != NULL; i++)
     {
-        size_t len = strlen(pronouns[i]);
         if (s.find(pronouns[i], start) == start)
         {
-            start += len;
+            start += strlen(pronouns[i]);
             break;
         }
     }
