@@ -1,4 +1,4 @@
-/*  $Id: FrenchVerbDictionary.cpp,v 1.56 2016/11/20 07:26:34 sarrazip Exp $
+/*  $Id: FrenchVerbDictionary.cpp,v 1.57 2017/08/29 01:47:43 sarrazip Exp $
     FrenchVerbDictionary.cpp - Dictionary of verbs and conjugation templates
 
     verbiste - French conjugation system
@@ -179,7 +179,6 @@ FrenchVerbDictionary::FrenchVerbDictionary(
                                 const string &verbsFilename,
                                 bool includeWithoutAccents,
                                 Language _lang)
-                                        throw (logic_error)
   : conjugSys(),
     knownVerbs(),
     aspirateHVerbs(),
@@ -196,7 +195,6 @@ FrenchVerbDictionary::FrenchVerbDictionary(
 
 
 FrenchVerbDictionary::FrenchVerbDictionary(bool includeWithoutAccents)
-                                                throw (std::logic_error)
   : conjugSys(),
     knownVerbs(),
     aspirateHVerbs(),
@@ -217,7 +215,6 @@ void
 FrenchVerbDictionary::init(const string &conjugationFilename,
                             const string &verbsFilename,
                             bool includeWithoutAccents)
-                                        throw (logic_error)
 {
     wideToUTF8Conv = iconv_open("UTF-8", "WCHAR_T");
     if (wideToUTF8Conv == (iconv_t) -1)
@@ -301,7 +298,6 @@ void
 FrenchVerbDictionary::loadConjugationDatabase(
                                 const char *conjugationFilename,
                                 bool includeWithoutAccents)
-                                        throw (logic_error)
 {
     if (conjugationFilename == NULL)
         throw invalid_argument("conjugationFilename");
@@ -318,7 +314,6 @@ void
 FrenchVerbDictionary::loadVerbDatabase(
                                 const char *verbsFilename,
                                 bool includeWithoutAccents)
-                                        throw (logic_error)
 {
     if (verbsFilename == NULL)
         throw invalid_argument("verbsFilename");
@@ -332,7 +327,7 @@ FrenchVerbDictionary::loadVerbDatabase(
 
 
 void
-FrenchVerbDictionary::readConjugation(xmlDocPtr doc, bool includeWithoutAccents) throw(logic_error)
+FrenchVerbDictionary::readConjugation(xmlDocPtr doc, bool includeWithoutAccents)
 {
     const bool isItalian = (lang == ITALIAN);
 
@@ -455,7 +450,6 @@ FrenchVerbDictionary::readConjugation(xmlDocPtr doc, bool includeWithoutAccents)
 
 string
 FrenchVerbDictionary::getUTF8XmlNodeText(xmlDocPtr doc, xmlNodePtr node)
-                                                                throw(int)
 {
     xmlChar *s = xmlNodeListGetString(doc, node, 1);
     if (s == NULL)
@@ -468,7 +462,6 @@ FrenchVerbDictionary::getUTF8XmlNodeText(xmlDocPtr doc, xmlNodePtr node)
 
 string
 FrenchVerbDictionary::getUTF8XmlProp(xmlNodePtr node, const char *propName)
-                                                                throw(int)
 {
     xmlChar *s = xmlGetProp(node, (const xmlChar *) propName);
     if (s == NULL)
@@ -485,7 +478,6 @@ FrenchVerbDictionary::getUTF8XmlProp(xmlNodePtr node, const char *propName)
 void
 FrenchVerbDictionary::readVerbs(xmlDocPtr doc,
                                 bool includeWithoutAccents)
-                                                throw(logic_error)
 {
     if (trace)
         cout << "readVerbs: start: includeWithoutAccents=" << includeWithoutAccents << endl;
@@ -837,7 +829,7 @@ void
 FrenchVerbDictionary::VerbTrie::onFoundPrefixWithUserData(
                         const wstring &conjugatedVerb,
                         wstring::size_type index,
-                        const vector<TrieValue> *templateList) const throw()
+                        const vector<TrieValue> *templateList) const
 {
     assert(templateList != NULL);
     if (trace)
@@ -1006,7 +998,7 @@ FrenchVerbDictionary::isWideVowel(wchar_t c)
 
 
 wstring
-FrenchVerbDictionary::utf8ToWide(const string &utf8String) const throw(int)
+FrenchVerbDictionary::utf8ToWide(const string &utf8String) const
 {
     size_t inbytesleft = utf8String.length() + 1;  // number of *bytes* in UTF-8 string
     size_t outbytesleft = inbytesleft * sizeof(wchar_t);  // oversized for safety
@@ -1044,7 +1036,7 @@ FrenchVerbDictionary::utf8ToWide(const string &utf8String) const throw(int)
 
 
 string
-FrenchVerbDictionary::wideToUTF8(const wstring &wideString) const throw(int)
+FrenchVerbDictionary::wideToUTF8(const wstring &wideString) const
 {
     size_t inbytesleft = (wideString.length() + 1) * sizeof(wchar_t);
     size_t outbytesleft = inbytesleft;  // UTF-8 string takes no more room than wstring
@@ -1072,7 +1064,7 @@ FrenchVerbDictionary::wideToUTF8(const wstring &wideString) const throw(int)
 string
 FrenchVerbDictionary::getRadical(
                         const string &infinitive,
-                        const string &templateName) throw(logic_error)
+                        const string &templateName)
 {
     string::size_type posColon = templateName.find(':');
     if (posColon == string::npos)
@@ -1092,7 +1084,7 @@ FrenchVerbDictionary::generateTense(const string &radical,
                                 vector< vector<string> > &dest,
                                 bool includePronouns,
                                 bool aspirateH,
-                                bool isItalian) const throw()
+                                bool isItalian) const
 {
     if (templ.find(mode) == templ.end())
         return false;
@@ -1176,7 +1168,7 @@ FrenchVerbDictionary::generateTense(const string &radical,
 
 
 bool FrenchVerbDictionary::isVerbStartingWithAspirateH(
-                                const std::string &infinitive) const throw()
+                                const std::string &infinitive) const
 {
     return aspirateHVerbs.find(infinitive) != aspirateHVerbs.end();
 }
